#!/bin/bash

function show_usage()
{
    cat >&2 <<EOF
Usage:
    $(basename $0) -f <file with pkglist>
    $(basename $0) pkg ...

    To also log dependencies to see their origin, use -d <output>

EOF
    exit 1
}

if [ -z "$MY_REPO" ]; then
    echo "Required environment variable undefined: MY_REPO" >&2
    exit 1
fi

STXTOOLS=${MY_REPO}/../stx-tools
REPOCFG=${STXTOOLS}/centos-mirror-tools/StarlingX.repo
RPMFILE=${STXTOOLS}/centos-mirror-tools/rpms_from_centos_repo.lst
WORKFILE=$(mktemp $(basename $0).XXXX)
SORTFILE=$(mktemp $(basename $0).XXXX)

PKGLIST=
DEPCHECK=

function cleanup()
{
    for f in $WORKFILE $SORTFILE; do
        test -f $f && rm -f $f
    done
}

trap cleanup EXIT

while getopts "d:f:h" opt; do
    case $opt in
        f)
            PKGLIST=$(cat $OPTARG)
            shift 2
            ;;
        d)
            DEPCHECK=$OPTARG
            shift 2
            ;;
        h)
            show_usage
            ;;
        *)
            echo "Unsupported option" >&2
            show_usage
            ;;
    esac
done

PKGLIST="${PKGLIST} $@"

function feature_to_pkg()
{
    local feature=$1
    local pkg=$(repoquery --quiet -c $REPOCFG $feature)
    if [ -z $pkg ]; then
        pkg=$(repoquery --quiet -c $REPOCFG --qf='%{name}' --whatprovides $feature | sort -u)
    fi
    if [ -z "$pkg" ]; then
        echo "Could not find in repo: $feature" >&2
    fi
    echo $pkg
}

function pkg_to_dependencies()
{
    local pkg=$1

    repoquery --quiet -c $REPOCFG --resolve --requires --recursive --qf='%{name}' $pkg
}

if [ -n "$DEPCHECK" ]; then
    echo "### $(date)" >> $DEPCHECK
fi

WORKFILE=$(mktemp $(basename $0).XXXX)
for p in $PKGLIST; do
    pkg=$(feature_to_pkg $p)
    if [ -z "$pkg" ]; then
        # Already msged to stderr
        continue
    fi

    dependencies=$(pkg_to_dependencies $pkg)
    echo $pkg >> $WORKFILE
    if [ -n "${dependencies}" ]; then
        echo "${dependencies}" >> $WORKFILE

        if [ -n "$DEPCHECK" ]; then
            echo "${dependencies}" | sed "s/^/$pkg::/" >> $DEPCHECK
        fi
    fi
done

sort -u $WORKFILE | while read pkg; do
    repoquery --quiet -c $REPOCFG --qf='%{relativepath}' $pkg | sed 's#.*/##'
done >> $RPMFILE

sort -u $RPMFILE > $SORTFILE
mv $SORTFILE $RPMFILE

